=== modified file 'HACKING'
--- HACKING	2005-12-16 00:17:28 +0000
+++ HACKING	2006-01-05 22:30:59 +0000
@@ -53,15 +53,40 @@
 Evolving interfaces
 -------------------
 
-If you change the behaviour of an API in an incompatible way, please
-be sure to change its name as well. For instance, if I add a keyword
-parameter to branch.commit - that's fine. On the other hand, if I add
-a keyword parameter to branch.commit which is a *required* transaction
-object, I should rename the API - i.e. to 'branch.commit_transaction'.
-
-This will prevent users of the old API getting surprising results. 
-Instead, they will get an Attribute error as the API is missing, and
-will know to update their code. If in doubt, just ask on #bzr.
+We have a commitment to 6 months API stability - any supported symbol in a
+release of bzr MUST NOT be altered in any way that would result in
+breaking existing code that uses it. That means that method names,
+parameter ordering, parameter names, variable and attribute names etc must
+not be changed without leaving a 'deprecated forwarder' behind. This even
+applies to modules and classes.
+
+If you wish to change the behaviour of a supported API in an incompatible
+way, you need to change its name as well. For instance, if I add a optional keyword
+parameter to branch.commit - that's fine. On the other hand, if I add a
+keyword parameter to branch.commit which is a *required* transaction
+object, I should rename the API - i.e. to 'branch.commit_transaction'. 
+
+When renaming such supported API's, be sure to leave a deprecated_method (or
+_function or ...) behind which forwards to the new API. See the
+bzrlib.symbol_versioning module for decorators that take care of the
+details for you - such as updating the docstring, and issuing a warning
+when the old api is used.
+
+For unsupported API's, it does not hurt to follow this discipline, but its
+not required. Minimally though, please try to rename things so that
+callers will at least get an AttributeError rather than weird results.
+
+
+Standard parameter types
+------------------------
+
+There are some common requirements in the library: some parameters need to be
+unicode safe, some need byte strings, and so on. At the moment we have
+only codified one specific pattern: Parameters that need to be unicode
+should be check via 'bzrlib.osutils.safe_unicode'. This will coerce the
+input into unicode in a consistent fashion, allowing trivial strings to be
+used for programmer convenience, but not performing unpredictably in the
+presence of different locales.
 
 Documentation
 =============

=== modified file 'NEWS'
--- NEWS	2006-01-02 22:54:45 +0000
+++ NEWS	2006-01-05 22:30:59 +0000
@@ -253,7 +253,14 @@
     * Simplify handling of DivergedBranches in cmd_pull().
       (Michael Ellerman)
 		   
-   
+    * Deprecated functions and methods can now be marked as such using the 
+      bzrlib.symbol_versioning module. Marked method have their docstring
+      updated and will issue a DeprecationWarning using the warnings module
+      when they are used. (Robert Collins)
+
+    * bzrlib.osutils.safe_unicode now exists to provide parameter coercion
+      for functions that need unicode strings. (Robert Collins)
+
 bzr 0.6 2005-10-28
 
   IMPROVEMENTS:

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-12-22 03:14:27 +0000
+++ bzrlib/__init__.py	2006-01-05 11:43:50 +0000
@@ -94,8 +94,11 @@
 version_info = (0, 7, 0, 'pre', 0)
 
 
+from bzrlib.symbol_versioning import deprecated_function, zero_seven
+
+@deprecated_function(zero_seven)
 def get_bzr_revision():
-    """If bzr is run from a branch, return (revno,revid) or None"""
+    """If bzr is run from a branch, return (revno,revid) or None."""
     import bzrlib.errors
     from bzrlib.branch import Branch
     

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-12-18 04:08:46 +0000
+++ bzrlib/builtins.py	2006-01-03 15:10:11 +0000
@@ -1461,12 +1461,31 @@
             bzrlib.ui.ui_factory = save_ui
 
 
+def _get_bzr_branch():
+    """If bzr is run from a branch, return Branch or None"""
+    import bzrlib.errors
+    from bzrlib.branch import Branch
+    from bzrlib.osutils import abspath
+    from os.path import dirname
+    
+    try:
+        branch = Branch.open(dirname(abspath(dirname(__file__))))
+        return branch
+    except bzrlib.errors.BzrError:
+        return None
+    
+
 def show_version():
     print "bzr (bazaar-ng) %s" % bzrlib.__version__
     # is bzrlib itself in a branch?
-    bzrrev = bzrlib.get_bzr_revision()
-    if bzrrev:
-        print "  (bzr checkout, revision %d {%s})" % bzrrev
+    branch = _get_bzr_branch()
+    if branch:
+        rh = branch.revision_history()
+        revno = len(rh)
+        print "  bzr checkout, revision %d" % (revno,)
+        print "  nick: %s" % (branch.nick,)
+        if rh:
+            print "  revid: %s" % (rh[-1],)
     print bzrlib.__copyright__
     print "http://bazaar-ng.org/"
     print

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-12-19 17:36:44 +0000
+++ bzrlib/errors.py	2006-01-05 22:30:59 +0000
@@ -34,8 +34,9 @@
 ... except:
 ...   print sys.exc_type
 ...   print sys.exc_value
-...   if hasattr(sys.exc_value, 'path'):
-...     print sys.exc_value.path
+...   path = getattr(sys.exc_value, 'path')
+...   if path is not None:
+...     print path
 bzrlib.errors.NotBranchError
 Not a branch: /foo/bar
 /foo/bar
@@ -447,12 +448,15 @@
         BzrNewError.__init__(self)
         self.graph = graph
 
+
 class NotConflicted(BzrNewError):
     """File %(filename)s is not conflicted."""
+
     def __init__(self, filename):
         BzrNewError.__init__(self)
         self.filename = filename
 
+
 class MustUseDecorated(Exception):
     """A decorating function has requested its original command be used.
     
@@ -461,8 +465,18 @@
 
 class MissingText(BzrNewError):
     """Branch %(base)s is missing revision %(text_revision)s of %(file_id)s"""
+
     def __init__(self, branch, text_revision, file_id):
+        BzrNewError.__init__(self)
         self.branch = branch
         self.base = branch.base
         self.text_revision = text_revision
         self.file_id = file_id
+
+
+class BzrBadParameter(BzrNewError):
+    """Parameter %(param)s is neither unicode nor utf8."""
+    
+    def __init__(self, param):
+        BzrNewError.__init__(self)
+        self.param = param

=== modified file 'bzrlib/export/dir_exporter.py'
--- bzrlib/export/dir_exporter.py	2005-11-14 23:58:51 +0000
+++ bzrlib/export/dir_exporter.py	2005-12-29 03:26:31 +0000
@@ -37,5 +37,10 @@
     mutter('export version %r', tree)
     inv = tree.inventory
     for dp, ie in inv.iter_entries():
+        # .bzrignore has no meaning outside of a working tree
+        # so do not export it
+        if dp == ".bzrignore":
+            continue
+        
         ie.put_on_disk(dest, dp, tree)
 

=== modified file 'bzrlib/export/tar_exporter.py'
--- bzrlib/export/tar_exporter.py	2005-11-14 23:58:51 +0000
+++ bzrlib/export/tar_exporter.py	2005-12-29 03:26:31 +0000
@@ -40,6 +40,11 @@
     mutter('export version %r', tree)
     inv = tree.inventory
     for dp, ie in inv.iter_entries():
+        # .bzrignore has no meaning outside of a working tree
+        # so do not export it
+        if dp == ".bzrignore":
+            continue
+        
         mutter("  export {%s} kind %s to %s", ie.file_id, ie.kind, dest)
         item, fileobj = ie.get_tar_item(root, dp, now, tree)
         ball.addfile(item, fileobj)

=== modified file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	2005-12-01 17:53:57 +0000
+++ bzrlib/export/zip_exporter.py	2006-01-02 21:20:36 +0000
@@ -39,6 +39,10 @@
 
     try:
         for dp, ie in inv.iter_entries():
+            # .bzrignore has no meaning outside of a working tree
+            # so do not export it
+            if dp == ".bzrignore":
+                continue
 
             file_id = ie.file_id
             mutter("  export {%s} kind %s to %s", file_id, ie.kind, dest)

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-01-02 22:37:32 +0000
+++ bzrlib/osutils.py	2006-01-05 22:30:59 +0000
@@ -31,7 +31,11 @@
 import tempfile
 
 import bzrlib
-from bzrlib.errors import BzrError, PathNotChild, NoSuchFile
+from bzrlib.errors import (BzrError,
+                           BzrBadParameter,
+                           NoSuchFile,
+                           PathNotChild,
+                           )
 from bzrlib.trace import mutter
 
 
@@ -142,7 +146,7 @@
         # RBC 20060103 abstraction leakage: the paramiko SFTP clients rename
         # function raises an IOError with errno == None when a rename fails.
         # This then gets caught here.
-        if e.errno is not None:
+        if e.errno not in (None, errno.ENOENT, errno.ENOTDIR):
             raise
     except Exception, e:
         if (not hasattr(e, 'errno') 
@@ -578,6 +582,22 @@
         return ''
 
 
+def safe_unicode(unicode_or_utf8_string):
+    """Coerce unicode_or_utf8_string into unicode.
+
+    If it is unicode, it is returned.
+    Otherwise it is decoded from utf-8. If a decoding error
+    occurs, it is wrapped as a If the decoding fails, the exception is wrapped 
+    as a BzrBadParameter exception.
+    """
+    if isinstance(unicode_or_utf8_string, unicode):
+        return unicode_or_utf8_string
+    try:
+        return unicode_or_utf8_string.decode('utf8')
+    except UnicodeDecodeError:
+        raise BzrBadParameter(unicode_or_utf8_string)
+
+
 def terminal_width():
     """Return estimated terminal width."""
 

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2005-10-17 23:13:00 +0000
+++ bzrlib/revisionspec.py	2005-12-30 20:54:47 +0000
@@ -204,7 +204,7 @@
         try:
             return RevisionInfo(branch, revs.index(self.spec) + 1, self.spec)
         except ValueError:
-            return RevisionInfo(branch, None)
+            return RevisionInfo(branch, None, self.spec)
 
 SPEC_TYPES.append(RevisionSpec_revid)
 

=== added file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	1970-01-01 00:00:00 +0000
+++ bzrlib/symbol_versioning.py	2006-01-05 21:48:56 +0000
@@ -0,0 +1,114 @@
+# Copyright (C) 2006 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Symbol versioning
+
+The methods here allow for api symbol versioning.
+"""
+
+__all__ = ['warn', 'set_warning_method', 'zero_seven']
+
+from warnings import warn
+
+
+zero_seven = "%s was deprecated in version 0.7."
+
+
+def set_warning_method(method):
+    """Set the warning method to be used by this module.
+
+    It should take a message and a warning category as warnings.warn does.
+    """
+    global warn
+    warn = method
+
+
+# TODO - maybe this would be easier to use as one 'smart' method that
+# guess if it is a method or a class or an attribute ? If so, we can
+# add that on top of the primitives, once we have all three written
+# - RBC 20050105
+
+def deprecated_function(deprecation_version):
+    """Decorate a function so that use of it will trigger a warning."""
+
+    def function_decorator(callable):
+        """This is the function python calls to perform the decoration."""
+        
+        def decorated_function(*args, **kwargs):
+            """This is the decorated function."""
+            symbol = "%s.%s" % (callable.__module__, 
+                                callable.__name__
+                                )
+            warn(deprecation_version % symbol, DeprecationWarning)
+            return callable(*args, **kwargs)
+        _populate_decorated(callable, deprecation_version, "function",
+                            decorated_function)
+        return decorated_function
+    return function_decorator
+
+
+def deprecated_method(deprecation_version):
+    """Decorate a method so that use of it will trigger a warning.
+    
+    To deprecate an entire class, decorate __init__.
+    """
+
+    def method_decorator(callable):
+        """This is the function python calls to perform the decoration."""
+        
+        def decorated_method(self, *args, **kwargs):
+            """This is the decorated method."""
+            symbol = "%s.%s.%s" % (self.__class__.__module__, 
+                                   self.__class__.__name__,
+                                   callable.__name__
+                                   )
+            warn(deprecation_version % symbol, DeprecationWarning)
+            return callable(self, *args, **kwargs)
+        _populate_decorated(callable, deprecation_version, "method",
+                            decorated_method)
+        return decorated_method
+    return method_decorator
+
+
+def _decorate_docstring(callable, deprecation_version, label,
+                        decorated_callable):
+    docstring_lines = callable.__doc__.split('\n')
+    if len(docstring_lines) == 0:
+        decorated_callable.__doc__ = deprecation_version % ("This " + label)
+    elif len(docstring_lines) == 1:
+        decorated_callable.__doc__ = (callable.__doc__ 
+                                    + "\n"
+                                    + "\n"
+                                    + deprecation_version % ("This " + label)
+                                    + "\n")
+    else:
+        spaces = len(docstring_lines[-1])
+        new_doc = callable.__doc__
+        new_doc += "\n" + " " * spaces
+        new_doc += deprecation_version % ("This " + label)
+        new_doc += "\n" + " " * spaces
+        decorated_callable.__doc__ = new_doc
+
+
+def _populate_decorated(callable, deprecation_version, label,
+                        decorated_callable):
+    """Populate attributes like __name__ and __doc__ on the decorated callable.
+    """
+    _decorate_docstring(callable, deprecation_version, label,
+                        decorated_callable)
+    decorated_callable.__module__ = callable.__module__
+    decorated_callable.__name__ = callable.__name__

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-01-05 09:01:32 +0000
+++ bzrlib/tests/__init__.py	2006-01-06 01:13:05 +0000
@@ -685,7 +685,6 @@
                    'bzrlib.tests.test_plugins',
                    'bzrlib.tests.test_remove',
                    'bzrlib.tests.test_revision',
-                   'bzrlib.tests.test_revision_info',
                    'bzrlib.tests.test_revisionnamespaces',
                    'bzrlib.tests.test_revprops',
                    'bzrlib.tests.test_reweave',
@@ -698,6 +697,7 @@
                    'bzrlib.tests.test_source',
                    'bzrlib.tests.test_status',
                    'bzrlib.tests.test_store',
+                   'bzrlib.tests.test_symbol_versioning',
                    'bzrlib.tests.test_testament',
                    'bzrlib.tests.test_trace',
                    'bzrlib.tests.test_transactions',

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2005-12-11 22:05:21 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-01-03 15:33:37 +0000
@@ -29,11 +29,13 @@
 def test_suite():
     testmod_names = [
                      'bzrlib.tests.blackbox.test_cat',
-                     'bzrlib.tests.blackbox.test_too_much',
+                     'bzrlib.tests.blackbox.test_export',
+                     'bzrlib.tests.blackbox.test_missing',
                      'bzrlib.tests.blackbox.test_pull',
                      'bzrlib.tests.blackbox.test_revno',
+                     'bzrlib.tests.blackbox.test_revision_info',
+                     'bzrlib.tests.blackbox.test_too_much',
                      'bzrlib.tests.blackbox.test_versioning',
-                     'bzrlib.tests.blackbox.test_missing'
                      ]
     return TestLoader().loadTestsFromNames(testmod_names)
 

=== added file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_export.py	2005-12-30 03:25:20 +0000
@@ -0,0 +1,74 @@
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr export.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+import tarfile, zipfile
+
+class TestExport(ExternalBase):
+
+    def test_tar_export(self):
+
+        os.mkdir('tar')
+        os.chdir('tar')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export test.tar.gz')
+        ball = tarfile.open('test.tar.gz')
+        for m in ball.getnames():
+            #print m
+            self.failIf(os.path.basename(m) == '.bzrignore', 'tar export contains .bzrignore')
+
+    def test_zip_export(self):
+
+        os.mkdir('zip')
+        os.chdir('zip')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export test.zip')
+        ball = zipfile.ZipFile('test.zip')
+        for m in ball.namelist():
+            #print m
+            self.failIf(os.path.basename(m) == '.bzrignore', 'zip export contains .bzrignore')
+
+    def test_dir_export(self):
+
+        os.mkdir('dir')
+        os.chdir('dir')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export direxport')
+
+        files = sorted(os.listdir('direxport'))
+        self.assertEqual(['a'], files, 'dir contains .bzrignore')

=== renamed file 'bzrlib/tests/test_revision_info.py' => 'bzrlib/tests/blackbox/test_revision_info.py'
=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2005-12-01 21:49:24 +0000
+++ bzrlib/tests/test_config.py	2005-12-30 22:03:42 +0000
@@ -507,14 +507,24 @@
         self.my_config._get_global_config()._get_parser(global_file)
 
     def test_set_user_setting_sets_and_saves(self):
-        # TODO RBC 20051029 test hat mkdir ~/.bazaar is called ..
         self.get_location_config('/a/c')
         record = InstrumentedConfigObj("foo")
         self.my_config._parser = record
-        print ("test_set_user_setting_sets_and_saves broken: creates .bazaar "
-               "in the top-level directory, not inside the test directory")
-        return
-        self.my_config.set_user_option('foo', 'bar')
+
+        real_mkdir = os.mkdir
+        self.created = False
+        def checked_mkdir(path, mode=0777):
+            self.log('making directory: %s', path)
+            real_mkdir(path, mode)
+            self.created = True
+
+        os.mkdir = checked_mkdir
+        try:
+            self.my_config.set_user_option('foo', 'bar')
+        finally:
+            os.mkdir = real_mkdir
+
+        self.failUnless(self.created, 'Failed to create ~/.bazaar')
         self.assertEqual([('__contains__', '/a/c'),
                           ('__contains__', '/a/c/'),
                           ('__setitem__', '/a/c', {}),

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-01-02 22:37:32 +0000
+++ bzrlib/tests/test_osutils.py	2006-01-05 22:44:46 +0000
@@ -21,8 +21,9 @@
 import sys
 
 import bzrlib
+from bzrlib.errors import BzrBadParameter
 import bzrlib.osutils as osutils
-from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir, TestCase
 
 
 class TestOSUtils(TestCaseInTempDir):
@@ -61,3 +62,20 @@
 
     # TODO: test fancy_rename using a MemoryTransport
 
+
+class TestSafeUnicode(TestCase):
+
+    def test_from_ascii_string(self):
+        self.assertEqual(u'foobar', osutils.safe_unicode('foobar'))
+
+    def test_from_unicode_string_ascii_contents(self):
+        self.assertEqual(u'bargam', osutils.safe_unicode(u'bargam'))
+
+    def test_from_unicode_string_unicode_contents(self):
+        self.assertEqual(u'bargam\xae', osutils.safe_unicode(u'bargam\xae'))
+
+    def test_from_utf8_string(self):
+        self.assertEqual(u'foo\xae', osutils.safe_unicode('foo\xc2\xae'))
+
+    def test_bad_utf8_string(self):
+        self.assertRaises(BzrBadParameter, osutils.safe_unicode, '\xbb\xbb')

=== modified file 'bzrlib/tests/test_revisionnamespaces.py'
--- bzrlib/tests/test_revisionnamespaces.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_revisionnamespaces.py	2005-12-30 20:54:47 +0000
@@ -80,6 +80,10 @@
         self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
                           'b@r-0-4')
 
+        # This should be in the revision store, but not in revision-history
+        self.assertEquals((None, 'b@r-0-4'),
+                RevisionSpec('revid:b@r-0-4').in_history(b))
+
     def test_branch_namespace(self):
         """Ensure that the branch namespace pulls in the requisite content."""
         self.build_tree(['branch1/', 'branch1/file', 'branch2/'])

=== modified file 'bzrlib/tests/test_sampler.py'
--- bzrlib/tests/test_sampler.py	2005-11-23 15:44:24 +0000
+++ bzrlib/tests/test_sampler.py	2006-01-05 10:49:26 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2006 by Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify

=== added file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2006-01-05 21:48:56 +0000
@@ -0,0 +1,91 @@
+# Copyright (C) 2006 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Symbol versioning tests."""
+
+import bzrlib.symbol_versioning as symbol_versioning
+from bzrlib.tests import TestCase
+
+
+@symbol_versioning.deprecated_function(symbol_versioning.zero_seven)
+def deprecated_function():
+    """Deprecated function docstring."""
+    return 1
+
+
+class TestDeprecationWarnings(TestCase):
+
+    def capture_warning(self, message, category):
+        self._warnings.append((message, category))
+
+    def setUp(self):
+        super(TestDeprecationWarnings, self).setUp()
+        self._warnings = []
+    
+    @symbol_versioning.deprecated_method(symbol_versioning.zero_seven)
+    def deprecated_method(self):
+        """Deprecated method docstring.
+        
+        This might explain stuff.
+        """
+        return 1
+
+    def test_deprecated_method(self):
+        expected_warning = (
+            "bzrlib.tests.test_symbol_versioning."
+            "TestDeprecationWarnings.deprecated_method "
+            "was deprecated in version 0.7.", DeprecationWarning)
+        expected_docstring = ('Deprecated method docstring.\n'
+                              '        \n'
+                              '        This might explain stuff.\n'
+                              '        \n'
+                              '        This method was deprecated in version 0.7.\n'
+                              '        ')
+        self.check_deprecated_callable(expected_warning, expected_docstring,
+                                       "deprecated_method",
+                                       "bzrlib.tests.test_symbol_versioning",
+                                       self.deprecated_method)
+
+    def test_deprecated_function(self):
+        expected_warning = (
+            "bzrlib.tests.test_symbol_versioning.deprecated_function "
+            "was deprecated in version 0.7.", DeprecationWarning)
+        expected_docstring = ('Deprecated function docstring.\n'
+                              '\n'
+                              'This function was deprecated in version 0.7.\n'
+                              )
+        self.check_deprecated_callable(expected_warning, expected_docstring,
+                                       "deprecated_function",
+                                       "bzrlib.tests.test_symbol_versioning",
+                                       deprecated_function)
+
+    def check_deprecated_callable(self, expected_warning, expected_docstring,
+                                  expected_name, expected_module,
+                                  deprecated_callable):
+        old_warning_method = symbol_versioning.warn
+        try:
+            symbol_versioning.set_warning_method(self.capture_warning)
+            self.assertEqual(1, deprecated_callable())
+            self.assertEqual([expected_warning], self._warnings)
+            deprecated_callable()
+            self.assertEqual([expected_warning, expected_warning],
+                             self._warnings)
+            self.assertEqualDiff(expected_docstring, deprecated_callable.__doc__)
+            self.assertEqualDiff(expected_name, deprecated_callable.__name__)
+            self.assertEqualDiff(expected_module, deprecated_callable.__module__)
+        finally:
+            symbol_versioning.set_warning_method(old_warning_method)

